@@ -1,4 +1,3 @@
-#! python
 # This module helps with debugging Frescobaldi.
 #
 # Start a Python shell
@@ -8,6 +7,7 @@
 # as app.
 
 from __future__ import unicode_literals
+from __future__ import print_function
 
 import sys
 
@@ -58,14 +58,13 @@ def modules():
     print('\n'.join(v.__name__ for k, v in sorted(sys.modules.items()) if v is not None))
 
 
-
-
 # avoid builtins._ being overwritten
 sys.displayhook = app.displayhook
 
 # instantiate app and create a mainwindow, etc
 app.instantiate()
 main.main()
+app.appStarted()
 
 # be friendly and import Qt stuff
 from PyQt4.QtCore import *
