@@ -23,9 +23,14 @@ Performs upgrades in the settings structure.
 
 from __future__ import unicode_literals
 
+try:
+    string_types = basestring
+except NameError:
+    string_types = str
+
 from PyQt4.QtCore import QSettings
 
-import info
+import appinfo
 
 def update(version):
     """Call subroutines listed below for version upgrades."""
@@ -39,10 +44,10 @@ def update(version):
 
 def moveSettingsToNewRoot():
     """Move all settings to one application file."""
-    movelist = [[info.name, info.url, False], "metainfo", "snippets", "sessions", "sessiondata"]
+    movelist = [[appinfo.name, appinfo.url, False], "metainfo", "snippets", "sessions", "sessiondata"]
     for moveitem in movelist:
-        if isinstance(moveitem, basestring):
-            moveitem = [moveitem, info.name, True]
+        if isinstance(moveitem, string_types):
+            moveitem = [moveitem, appinfo.name, True]
         o = QSettings(moveitem[1], moveitem[0])
         o.setFallbacksEnabled(False)
         keys = o.allKeys()
