@@ -21,6 +21,7 @@
 Frescobaldi Main Window.
 """
 
+from __future__ import division
 from __future__ import unicode_literals
 
 import itertools
@@ -28,12 +29,17 @@ import os
 import sys
 import weakref
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from PyQt4.QtCore import (pyqtSignal, QByteArray, QDir, QMimeData, QSettings,
+                          QSize, Qt, QUrl)
+from PyQt4.QtGui import (QAbstractPrintDialog, QAction, QApplication,
+                         QFileDialog, QKeySequence, QMainWindow, QMenu,
+                         QMessageBox, QPlainTextEdit, QPrintDialog, QPrinter,
+                         QTextCursor, QTextDocument, QVBoxLayout, QWhatsThis,
+                         QWidget)
 
 import app
 import backup
-import info
+import appinfo
 import icons
 import actioncollection
 import actioncollectionmanager
@@ -55,6 +61,7 @@ import scorewiz
 import externalchanges
 import browseriface
 import vcs
+import file_import
 
 
 class MainWindow(QMainWindow):
@@ -180,10 +187,13 @@ class MainWindow(QMainWindow):
         """Switch to the cursor's document() and set that cursor on its View.
         
         For the findOpenView argument, see setCurrentDocument().
+        This method also respects the preferred number of surrounding lines
+        that are at least to be shown (by using the gotoTextCursor() method of
+        the View (see view.py)).
         
         """
         self.setCurrentDocument(cursor.document(), findOpenView)
-        self.currentView().setTextCursor(cursor)
+        self.currentView().gotoTextCursor(cursor)
     
     def slotViewChanged(self, view):
         curv = self._currentView()
@@ -266,7 +276,19 @@ class MainWindow(QMainWindow):
     def dropEvent(self, ev):
         if not ev.source() and ev.mimeData().hasUrls():
             ev.accept()
-            self.openDocuments(ev.mimeData().urls())
+            lyurls = []
+            impurls = []
+            for url in ev.mimeData().urls():
+                imp = file_import.FileImport.instance(self)
+                if imp.isImportable(url.toLocalFile()):
+                    impurls.append(QDir.toNativeSeparators(url.toLocalFile()))
+                else:
+                    lyurls.append(url)
+            docs = self.openUrls(lyurls)
+            if docs:
+                self.setCurrentDocument(docs[-1])
+            for i in impurls:
+                imp.openDialog(i)
     
     def dragEnterEvent(self, ev):
         if not ev.source() and ev.mimeData().hasUrls():
@@ -368,28 +390,39 @@ class MainWindow(QMainWindow):
         settings.setValue('geometry', self.saveGeometry())
         settings.setValue('state', self.saveState())
 
-    def openUrl(self, url, encoding=None):
-        """Same as app.openUrl but adds url to recent files."""
-        d = app.openUrl(url, encoding)
-        recentfiles.add(url)
-        return d
+    def openUrl(self, url, encoding=None, ignore_errors=False):
+        """Calls openUrls() with one url. See openUrls()."""
+        for d in self.openUrls([url], encoding, ignore_errors):
+            return d
     
-    def openDocuments(self, urls, encoding=None):
-        """Open a list of urls, with error handling.
+    def openUrls(self, urls, encoding=None, ignore_errors=False):
+        """Open a list of urls, using encoding if specified.
         
-        The last loaded document is made current.
+        Returns the list of documents that were successfully loaded.
+        
+        If encoding is None, the encoding is read from the document, defaulting
+        to UTF-8.
+        
+        If ignore_errors is False (the default), an error message is given
+        showing the url or urls that failed to load. If ignore_errors is True,
+        load errors are silently ignored.
+        
+        If an url fails to load, a document is not created. To create an
+        empty document with an url, use the document.Document constructor.
+        Successfully loaded urls are added to the recent files.
         
         """
-        doc = None
+        docs = []
         failures = []
         for url in urls:
             try:
-                doc = self.openUrl(url)
+                doc = app.openUrl(url, encoding)
             except IOError as e:
                 failures.append((url, e))
-        if doc:
-            self.setCurrentDocument(doc)
-        if failures:
+            else:
+                docs.append(doc)
+                recentfiles.add(url)
+        if failures and not ignore_errors:
             if len(failures) == 1:
                 url, e = failures[0]
                 filename = url.toLocalFile()
@@ -404,7 +437,8 @@ class MainWindow(QMainWindow):
                         strerror = e.strerror,
                         errno = e.errno) for url, e in failures)
             QMessageBox.critical(self, app.caption(_("Error")), msg)
-                
+        return docs
+    
     def currentDirectory(self):
         """Returns the current directory of the current document.
         
@@ -415,8 +449,14 @@ class MainWindow(QMainWindow):
         
         """
         import resultfiles
-        return (resultfiles.results(self.currentDocument()).currentDirectory()
-                or app.basedir() or QDir.homePath() or os.getcwdu())
+        curdir = (resultfiles.results(self.currentDocument()).currentDirectory()
+                  or app.basedir() or QDir.homePath())
+        if curdir:
+            return curdir
+        try:
+            return os.getcwdu()
+        except AttributeError:
+            return os.getcwd()
     
     def cleanStart(self):
         """Called when the previous action left no document open.
@@ -457,7 +497,9 @@ class MainWindow(QMainWindow):
         directory = os.path.dirname(self.currentDocument().url().toLocalFile()) or app.basedir()
         files = QFileDialog.getOpenFileNames(self, caption, directory, filetypes)
         urls = [QUrl.fromLocalFile(filename) for filename in files]
-        self.openDocuments(urls)
+        docs = self.openUrls(urls)
+        if docs:
+            self.setCurrentDocument(docs[-1])
         
     def saveDocument(self, doc, save_as=False):
         """ Saves the document, asking for a name if necessary.
@@ -546,7 +588,7 @@ class MainWindow(QMainWindow):
             import fileinfo
             text = self.currentView().textCursor().selection().toPlainText()
             mode = fileinfo.textmode(text)
-            data = util.encode(text)
+            data = util.encode(util.platform_newlines(text))
             caption = app.caption(_("dialog title", "Save Selection"))
         filetypes = app.filetypes(ly.lex.extensions[mode])
         dirname = os.path.dirname(doc.url().toLocalFile()) or app.basedir()
@@ -554,7 +596,7 @@ class MainWindow(QMainWindow):
         if not filename:
             return # cancelled
         try:
-            with open(filename, "w") as f:
+            with open(filename, "wb") as f:
                 f.write(data)
         except IOError as e:
             msg = _("{message}\n\n{strerror} ({errno})").format(
@@ -666,7 +708,7 @@ class MainWindow(QMainWindow):
         filename = QFileDialog.getOpenFileName(self, caption, directory, filetypes)
         if filename:
             try:
-                with open(filename) as f:
+                with open(filename, 'rb') as f:
                     data = f.read()
             except IOError as e:
                 msg = _("{message}\n\n{strerror} ({errno})").format(
@@ -675,7 +717,7 @@ class MainWindow(QMainWindow):
                     errno = e.errno)
                 QMessageBox.critical(self, app.caption(_("Error")), msg)
             else:
-                text = util.decode(data)
+                text = util.universal_newlines(util.decode(data))
                 self.currentView().textCursor().insertText(text)
         
     def openCurrentDirectory(self):
@@ -686,7 +728,12 @@ class MainWindow(QMainWindow):
     
     def printSource(self):
         cursor = self.textCursor()
-        printer = QPrinter()
+        try:
+            printer = self._sourcePrinter
+        except AttributeError:
+            printer = self._sourcePrinter = QPrinter()
+        else:
+            printer.setCopyCount(1)
         dlg = QPrintDialog(printer, self)
         dlg.setWindowTitle(app.caption(_("dialog title", "Print Source")))
         options = QAbstractPrintDialog.PrintToFile | QAbstractPrintDialog.PrintShowPageSize
@@ -924,8 +971,9 @@ class MainWindow(QMainWindow):
     
     def slotRecentFilesAction(self, action):
         """Called when a recent files menu action is triggered."""
-        doc = self.openUrl(action.url)
-        self.setCurrentDocument(doc)
+        d = self.openUrl(action.url)
+        if d:
+            self.setCurrentDocument(d)
         
     def createMenus(self):
         menu.createMenus(self)
@@ -943,7 +991,8 @@ class MainWindow(QMainWindow):
         t.addAction(ac.file_new)
         t.addAction(ac.file_open)
         t.addAction(ac.file_save)
-        t.addAction(ac.file_save_as)
+        t.widgetForAction(ac.file_save).addAction(ac.file_save_as)
+        t.widgetForAction(ac.file_save).addAction(ac.file_save_all)
         t.addAction(ac.file_close)
         t.addSeparator()
         t.addAction(browseriface.get(self).actionCollection.go_back)
@@ -954,6 +1003,9 @@ class MainWindow(QMainWindow):
         t.addSeparator()
         t.addAction(scorewiz.ScoreWizard.instance(self).actionCollection.scorewiz)
         t.addAction(engrave.engraver(self).actionCollection.engrave_runner)
+        w = t.widgetForAction(engrave.engraver(self).actionCollection.engrave_runner)
+        w.addAction(engrave.engraver(self).actionCollection.engrave_publish)
+        w.addAction(engrave.engraver(self).actionCollection.engrave_custom)
         
         self.toolbar_music = t = self.addToolBar('')
         t.setObjectName('toolbar_music')
@@ -1139,7 +1191,7 @@ class ActionCollection(actioncollection.ActionCollection):
         self.file_close_all.setText(_("Close All Documents and Session"))
         self.file_close_all.setToolTip(_("Closes all documents and leaves the current session."))
         self.file_quit.setText(_("&Quit"))
-        self.file_restart.setText(_("Restart {appname}").format(appname=info.appname))
+        self.file_restart.setText(_("Restart {appname}").format(appname=appinfo.appname))
         
         self.export_colored_html.setText(_("Export Source as Colored &HTML..."))
         
@@ -1172,6 +1224,6 @@ class ActionCollection(actioncollection.ActionCollection):
         self.help_manual.setText(_("&User Guide"))
         self.help_whatsthis.setText(_("&What's This?"))
         self.help_bugreport.setText(_("Report a &Bug..."))
-        self.help_about.setText(_("&About {appname}...").format(appname=info.appname))
+        self.help_about.setText(_("&About {appname}...").format(appname=appinfo.appname))
         
 
