@@ -22,7 +22,8 @@
 Surface is the widget everything is drawn on.
 """
 
-import itertools
+import collections
+import operator
 import weakref
 
 from PyQt4.QtCore import QEvent, QPoint, QRect, QSize, Qt, QTimer, pyqtSignal
@@ -83,8 +84,8 @@ class CustomRubberBand(QWidget):
         region += QRect(self.rect().width()-20, self.rect().height()-20, 20, 20)
         region += QRect(0, self.rect().height()-20, 20, 20)
         # Clip middles
-        region += QRect(0, self.rect().height()/2-10, self.rect().width(), 20)
-        region += QRect(self.rect().width()/2-10, 0, 20, self.rect().height())
+        region += QRect(0, self.rect().height() // 2 - 10, self.rect().width(), 20)
+        region += QRect(self.rect().width() // 2 - 10, 0, 20, self.rect().height())
         
         # Draw thicker rectangles, clipped at corners and sides.
         painter.setClipRegion(region)
@@ -182,7 +183,7 @@ class Surface(QWidget):
         """
         if self._magnifier:
             self._magnifier.setParent(None)
-        magnifier.setParent(self)
+        magnifier.setParent(self.view())
         self._magnifier = magnifier
     
     def magnifier(self):
@@ -239,9 +240,10 @@ class Surface(QWidget):
         linkArea attribute of a Poppler.Link.
         
         """
-        d = weakref.WeakKeyDictionary()
-        for page, areas in itertools.groupby(sorted(areas), lambda a: a[0]):
-            d[page] = list(area[1] for area in areas)
+        d = collections.defaultdict(list)
+        for page, area in areas:
+            d[page].append(area)
+        d = weakref.WeakKeyDictionary(d)
         if msec:
             def clear(selfref=weakref.ref(self)):
                 self = selfref()
@@ -507,7 +509,7 @@ class Surface(QWidget):
         
         (Use this instead of the contextMenuEvent as that one also
         fires when starting a right-button selection.)
-        The default implementation emite the rightClicked(pos) signal and also
+        The default implementation emits the rightClicked(pos) signal and also
         sends a ContextMenu event to the View widget.
         
         """
