@@ -29,10 +29,9 @@ import re
 
 import ly.dom
 import po.mofile
+import lasptyqu
 
 from . import parts
-import parts._base
-import parts.containers
 
 
 class PartNode(object):
@@ -62,7 +61,7 @@ class PartNode(object):
 
 
 class PartData(object):
-    """Represents what a Part wants to add to the LilyPond score.
+    r"""Represents what a Part wants to add to the LilyPond score.
     
     A Part may append to the following instance attributes (which are lists):
     
@@ -199,6 +198,11 @@ class Builder(object):
         p = self._printer = ly.dom.Printer()
         p.indentString = "  " # will be re-indented anyway
         p.typographicalQuotes = generalPreferences.typq.isChecked()
+        quotes = lasptyqu.preferred()
+        p.primary_quote_left = quotes.primary.left
+        p.primary_quote_right = quotes.primary.right
+        p.secondary_quote_left = quotes.secondary.left
+        p.secondary_quote_right = quotes.secondary.right
         if self.pitchLanguage:
             p.language = self.pitchLanguage
         
